@@ -12,33 +12,29 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import os
 from collections.abc import Iterable, Iterator, Mapping, Sequence
-from dataclasses import dataclass, field
+from dataclasses import asdict, dataclass, field
 from functools import partial
-from typing import Any, Callable, Dict, Generator, List, Optional, Tuple, Union
+from typing import Any, Callable, Dict, List, Optional, Union
 
 import torch
-from torch import Tensor
 from torch.utils.data import Dataset
 from torch.utils.data.dataloader import _BaseDataLoaderIter, _MultiProcessingDataLoaderIter, DataLoader
 from torch.utils.data.dataset import IterableDataset
 
 from pytorch_lightning.utilities.apply_func import apply_to_collection, apply_to_collections
 from pytorch_lightning.utilities.auto_restart import (
-    _cycle_to_next_worker_and_reset,
-    _find_current_worker,
-    CaptureIterableDataset,
+    MergedIteratorState,
+    patch_dataloader_iterator,
+    reload_dataloader_state_dict,
 )
-from pytorch_lightning.utilities.cloud_io import get_filesystem
 from pytorch_lightning.utilities.data import get_len
 from pytorch_lightning.utilities.exceptions import MisconfigurationException
-from pytorch_lightning.utilities.imports import _fault_tolerant_enabled
+from pytorch_lightning.utilities.imports import _fault_tolerant_training
 
 
 class TensorRunningAccum:
-    """Tracks a running accumulation values (min, max, mean) without graph
-    references.
+    """Tracks a running accumulation values (min, max, mean) without graph references.
 
     Examples:
         >>> accum = TensorRunningAccum(5)
@@ -113,66 +109,14 @@ class TensorRunningAccum:
             return getattr(self.memory[: self.current_idx], how)()
 
 
-class PredictionCollection:
-    def __init__(self, global_rank: int, world_size: int):
-        self.global_rank = global_rank
-        self.world_size = world_size
-        self.predictions = {}
-        self.num_predictions = 0
-
-    def _add_prediction(self, name, values, filename):
-        if filename not in self.predictions:
-            self.predictions[filename] = {name: values}
-        elif name not in self.predictions[filename]:
-            self.predictions[filename][name] = values
-        elif isinstance(values, Tensor):
-            self.predictions[filename][name] = torch.cat((self.predictions[filename][name], values))
-        elif isinstance(values, list):
-            self.predictions[filename][name].extend(values)
-
-    def add(self, predictions):
-
-        if predictions is None:
-            return
-
-        for filename, pred_dict in predictions.items():
-            for feature_name, values in pred_dict.items():
-                self._add_prediction(feature_name, values, filename)
-
-    def to_disk(self) -> None:
-        """Write predictions to file(s)."""
-        for filepath, predictions in self.predictions.items():
-            fs = get_filesystem(filepath)
-            # normalize local filepaths only
-            if fs.protocol == "file":
-                filepath = os.path.realpath(filepath)
-            if self.world_size > 1:
-                stem, extension = os.path.splitext(filepath)
-                filepath = f"{stem}_rank_{self.global_rank}{extension}"
-            dirpath = os.path.split(filepath)[0]
-            fs.mkdirs(dirpath, exist_ok=True)
-
-            # Convert any tensor values to list
-            predictions = {k: v if not isinstance(v, Tensor) else v.tolist() for k, v in predictions.items()}
-
-            # Check if all features for this file add up to same length
-            feature_lens = {k: len(v) for k, v in predictions.items()}
-            if len(set(feature_lens.values())) != 1:
-                raise ValueError("Mismatching feature column lengths found in stored EvalResult predictions.")
-
-            # Switch predictions so each entry has its own dict
-            outputs = []
-            for values in zip(*predictions.values()):
-                output_element = dict(zip(predictions.keys(), values))
-                outputs.append(output_element)
-
-            # Write predictions for current file to disk
-            with fs.open(filepath, "wb") as fp:
-                torch.save(outputs, fp)
-
-
 @dataclass
 class SharedCycleIteratorState:
+    """A state shared between all CylceIterators in a CombinedLoader.
+
+    With a shared state, the iterators can decide to terminate based on the state of all others. If the mode is
+    *max_size_cycle*, all iterators need to have finished before the combined loading is considered finished, and
+    otherwise any iterator finishing early will lead to all iterators ending early.
+    """
 
     mode: str = "max_size_cycle"
     dataloaders: List[DataLoader] = field(default_factory=lambda: [])
@@ -187,7 +131,7 @@ class SharedCycleIteratorState:
     @property
     def done(self) -> bool:
         if not self.has_reset:
-            raise MisconfigurationException("Please, call reset once all dataloaders have been added.")
+            raise MisconfigurationException("Please call reset once all dataloaders have been added.")
         if len(self.dataloaders) == 1:
             return False
         decision_fn = all if self.mode == "max_size_cycle" else any
@@ -195,9 +139,7 @@ class SharedCycleIteratorState:
 
 
 class CycleIterator:
-    """
-    Iterator for restarting a dataloader if it runs out of samples
-    """
+    """Iterator for restarting a dataloader if it runs out of samples."""
 
     def __init__(self, loader: Any, length: Optional[int] = None, state: SharedCycleIteratorState = None):
         """
@@ -222,15 +164,16 @@ class CycleIterator:
         self.loader = loader
         self._loader_iter = None
         self.counter = 0
+        self.state = state
 
     def __iter__(self) -> Any:
-        """
-        Creates the internal iterator and returns self
+        """Creates the internal iterator and returns self.
 
         Returns:
             CycleIterator: self
         """
         self.counter = 0
+        self.state.reset()
         self._loader_iter = iter(self.loader)
         return self
 
@@ -260,6 +203,12 @@ class CycleIterator:
                 raise StopIteration
 
             self._loader_iter = iter(self.loader)
+            # if fault tolerant is enabled, we need to patch the iterator to collect the states
+            # before the batch gets returned.
+            fetcher = getattr(self.loader, "_lightning_fetcher", None)
+            if fetcher:
+                patch_dataloader_iterator(self.loader, self._loader_iter, fetcher)
+
             return next(self._loader_iter)
 
         finally:
@@ -270,9 +219,7 @@ class CycleIterator:
 
 
 class CombinedDataset:
-    """
-    Combine multiple datasets and compute their statistics
-    """
+    """Combine multiple datasets and compute their statistics."""
 
     COMPUTE_FUNCS = {"min_size": min, "max_size_cycle": max}
 
@@ -301,8 +248,7 @@ class CombinedDataset:
         return self._calc_num_data(self.datasets, "min_size")
 
     def _calc_num_data(self, datasets: Union[Sequence, Mapping], mode: str) -> Union[int, float]:
-        """
-        Compute the length of `CombinedDataset` according to the `mode`.
+        """Compute the length of `CombinedDataset` according to the `mode`.
 
         Args:
             datasets: a sequence/mapping datasets. Can be a collections of torch.utils.data.Dataset,
@@ -357,18 +303,11 @@ class CombinedDataset:
         return self._calc_num_data(self.datasets, self.mode)
 
 
-class DataLoaderDict(Dict):
-    # behaves exactly like a dict, this is used to simplify apply_to_collection.
-    pass
-
-
 class CombinedLoader:
-    """
-    Combines different dataloaders and allows sampling in parallel.
-    Supported modes are 'min_size', which raises StopIteration after the shortest loader
-    (the one with the lowest number of batches) is done, and 'max_size_cycle` which raises
-    StopIteration after the longest loader (the one with most batches) is done, while cycling
-    through the shorter loaders.
+    """Combines different dataloaders and allows sampling in parallel. Supported modes are 'min_size', which raises
+    StopIteration after the shortest loader (the one with the lowest number of batches) is done, and
+    'max_size_cycle` which raises StopIteration after the longest loader (the one with most batches) is done, while
+    cycling through the shorter loaders.
 
     Examples:
         >>> loaders = {'a': torch.utils.data.DataLoader(range(6), batch_size=4),
@@ -415,88 +354,92 @@ class CombinedLoader:
         self._iterator = None  # assigned in __iter__
 
     @staticmethod
-    def _state_dict_fn(dataloader: DataLoader, iterator: Optional[Iterator], num_batches_processed: int) -> Dict:
-        # find next worker if multiple workers were used
-        state = _find_current_worker(iterator)
-        if isinstance(dataloader.dataset, CaptureIterableDataset):
-            # the sampler state dict are extracted in `CombinedLoaderIterator`
-            if iterator is not None and getattr(iterator, "_sampler_state_dict", None) is not None:
-                state.update(iterator._sampler_state_dict[0])
-        else:
-            # fetch directly from fast forward sampler
-            state.update(dataloader.fast_forward_sampler.state_dict(num_batches_processed))
-        return DataLoaderDict(state)
+    def _state_dict_fn(dataloader: DataLoader, iterator: Optional[Iterator], has_completed: int) -> Dict:
+        if isinstance(dataloader, CycleIterator):
+            iterator = dataloader._loader_iter
 
-    def state_dict(self, num_batches_processed: int) -> Dict:
-        """
-        The state dict includes all states from wrapped dataloaders and their samplers through the
+        # There is currently 2 dataloader states being tracked: (batch_n - 1, state_n - 1), (batch_n, state_n)
+        # where `n` is the current batch. If the batch was processed, it should be saved to reproduce the next batch.
+        # Otherwise, we want to get the state of the previous batch, so we can reproduce the current batch.
+        # The state is stored directly on the Iterator as an attribute by the DataFetcher for accessibility
+        state_to_save = "state" if has_completed else "previous_state"
+        state: Optional[MergedIteratorState] = getattr(iterator, state_to_save, None)
+        if state:
+            return asdict(state)
+        return {}
+
+    def state_dict(self, has_completed: bool = False) -> Dict:
+        """The state dict includes all states from wrapped dataloaders and their samplers through the
         ``CaptureIterableDataset`` and fast-forward samplers.
 
         Args:
-            num_batches_processed: The number of batches processed so far, needed because the individual dataloaders
-                may have already prefetched more batches by the time a state dict is requested.
+            has_completed: whether the current state of data fetching is considered completed or not. If it is, the
+                current state gets returned, otherwise the previously cached state.
         """
-        if not _fault_tolerant_enabled():
-            return DataLoaderDict()
+        if not _fault_tolerant_training() or self._iterator is None:
+            return {}
 
-        state_dict_fn = partial(self._state_dict_fn, num_batches_processed=num_batches_processed)
+        return apply_to_collections(
+            self.loaders,
+            self._iterator.loader_iters,
+            (Iterator, DataLoader),
+            partial(self._state_dict_fn, has_completed=has_completed),
+        )
 
-        return apply_to_collections(self.loaders, self._iterator.loader_iters, (Iterator, DataLoader), state_dict_fn)
-
-    def load_state_dict(self, state_dict):
+    def load_state_dict(self, state_dict) -> None:
         # store the samplers state.
         # They would be reloaded once the `CombinedIterator` as been created
         # and the workers are created.
         self._loaders_iter_state_dict = state_dict
 
-        def mock_reset_fn(self, *_, **__):
-            pass
-
-        # mock reset call, so we can rotate the `_worker_queue_idx_cycle` to failed worker
-        # and get the first batch from it
-        _MultiProcessingDataLoaderIter._original_reset = _MultiProcessingDataLoaderIter._reset
-        _MultiProcessingDataLoaderIter._reset = mock_reset_fn
-
-    def on_restart(self, iterator: Iterator):
+    def on_restart(self, iterator: Iterator) -> None:
         if not self._loaders_iter_state_dict:
             return
 
-        # this happen inside the workers if any were specificied.
+        def create_loader_iters(dataloader: DataLoader, state_dict: Dict) -> Iterator:
+            """Function used to reload the iterator state before once the workers are created."""
 
-        def create_loader_iters(dataloader: DataLoader, state_dict: DataLoaderDict):
-            if isinstance(dataloader.dataset, CaptureIterableDataset):
-                # provide the `state_dict` to the `CaptureIterableDataset`
-                # as it is responsible for passing down the state to associated `FastForwardSampler`
-                dataloader.dataset.load_state_dict(state_dict)
+            dataloader_to_iter_on = dataloader
+            if isinstance(dataloader, CycleIterator):
+                dataloader = dataloader_to_iter_on.loader
+
+            reload_dataloader_state_dict(dataloader, state_dict)
+
+            # We finally spawned the workers if any.
+            it = iter(dataloader_to_iter_on)
+
+            # restore caching state
+            state = MergedIteratorState.from_state_dict(state_dict)
+
+            if isinstance(dataloader_to_iter_on, CycleIterator):
+                it._loader_iter.state = state
             else:
-                # for `Mapping-based` dataset, the `fast_forward_sampler` was attached
-                # on the dataloader for simplicity
-                dataloader.fast_forward_sampler.load_state_dict(state_dict)
+                it.state = state
+            return it
 
-            # cycle back the iterator to the failed worker if multiple workers were provided
-            iterator = _cycle_to_next_worker_and_reset(dataloader, state_dict)
-
-            if isinstance(dataloader.dataset, CaptureIterableDataset):
-                # remove keys related to iterator
-                state_dict = {k: v for k, v in state_dict.items() if k not in ("num_worker", "previous_worker")}
-                # need to re-attach the state dict into the iterator for future collection.
-                iterator._sampler_state_dict = [state_dict]
-            return iterator
+        # create an un-existing token, so it doesn't activate for something else than an iterator.
+        class DataLoaderDict(dict):
+            pass
 
         # apply the `create_loader_iters` on the collection of `DataLoader / Iterator`.
         # each `Iterator` was created from the `DataLoader`.
         iterator._loader_iters = apply_to_collections(
-            self.loaders, self._loaders_iter_state_dict, (DataLoader, DataLoaderDict), create_loader_iters
+            self.loaders,
+            self._loaders_iter_state_dict,
+            (Iterable, DataLoaderDict),
+            create_loader_iters,
+            wrong_dtype=(Sequence, Mapping),
         )
 
+        self._loaders_iter_state_dict = None
+
     @property
     def sampler(self) -> Union[Iterable, Sequence, Mapping]:
         """Return a collections of samplers extracting from loaders."""
         return apply_to_collection(self.loaders, (DataLoader, IterableDataset), getattr, "sampler", None)
 
     def _wrap_loaders_max_size_cycle(self) -> Any:
-        """
-        Wraps all loaders to make sure they are cycled until the longest loader is exhausted
+        """Wraps all loaders to make sure they are cycled until the longest loader is exhausted.
 
         Returns:
             the wrapped loaders
@@ -512,13 +455,10 @@ class CombinedLoader:
             self.loaders = apply_to_collection(
                 self.loaders, Iterable, CycleIterator, length=length, state=state, wrong_dtype=(Sequence, Mapping)
             )
-
             state.reset()
 
     def __iter__(self) -> Any:
-        """
-        Create and return an iterator, `CombinedLoaderIterator`, for the combined loader.
-        """
+        """Create and return an iterator, `CombinedLoaderIterator`, for the combined loader."""
 
         # prevent `NotImplementedError` from PyTorch:
         # https://github.com/pytorch/pytorch/blob/v1.9.0/torch/utils/data/dataloader.py#L541
@@ -534,8 +474,7 @@ class CombinedLoader:
 
     @staticmethod
     def _calc_num_batches(loaders: Any) -> Union[int, float]:
-        """
-        Compute the length (aka the number of batches) of `CombinedLoader`.
+        """Compute the length (aka the number of batches) of `CombinedLoader`.
 
         Args:
             loaders: a collections of loaders.
@@ -552,11 +491,22 @@ class CombinedLoader:
     def __len__(self) -> int:
         return self._calc_num_batches(self.loaders)
 
+    @staticmethod
+    def _shutdown_workers_and_reset_iterator(dataloader) -> None:
+        if hasattr(dataloader, "_iterator") and isinstance(dataloader._iterator, _MultiProcessingDataLoaderIter):
+            dataloader._iterator._shutdown_workers()
+        dataloader._iterator = None
+
+    def reset(self):
+        if self._iterator:
+            self._iterator._loader_iters = None
+        if self.loaders is not None:
+            apply_to_collection(self.loaders, DataLoader, self._shutdown_workers_and_reset_iterator)
+        self._iterator = None
+
 
 class CombinedLoaderIterator:
-    """
-    Custom Iterator returning data from multple loaders, and allows sampling in parallel
-    """
+    """Custom Iterator returning data from multple loaders, and allows sampling in parallel."""
 
     def __init__(self, loaders: Any):
         """
@@ -568,9 +518,7 @@ class CombinedLoaderIterator:
 
     @property
     def loader_iters(self) -> Any:
-        """
-        Get the `_loader_iters` and create one if it is None.
-        """
+        """Get the `_loader_iters` and create one if it is None."""
         if self._loader_iters is None:
             self._loader_iters = self.create_loader_iters(self.loaders)
 
@@ -580,8 +528,7 @@ class CombinedLoaderIterator:
         return self
 
     def __next__(self) -> Any:
-        """
-        Fetches the next batch from multiple data loaders
+        """Fetches the next batch from multiple data loaders.
 
         Returns:
             a collections of batch data
@@ -590,8 +537,7 @@ class CombinedLoaderIterator:
 
     @staticmethod
     def request_next_batch(loader_iters: Union[Iterator, Sequence, Mapping]) -> Any:
-        """
-        Return the batch of data from multiple iterators.
+        """Return the batch of data from multiple iterators.
 
         Args:
             loader_iters: a collections of iterators
@@ -599,29 +545,13 @@ class CombinedLoaderIterator:
         Returns
             Any: a collections of batch data
         """
-
-        def next_fn(iterator: Iterator):
-            batch = next(iterator)
-            if not _fault_tolerant_enabled():
-                return batch
-            # when fault tolerant is enabled, the iterator will return
-            # `FastForwardSampler` state_dict metadata
-            # along side with the user data.
-            # the metadata are extracted and store directly on the iterator
-            # to simplify the collection on `state_dict` call.
-            batch, samplers_state_dict = CaptureIterableDataset.extract_samplers_state_dict_from_batch(batch)
-            # store the `sampler_state_dict` on the iterator
-            CaptureIterableDataset.store_samplers_state_dict(iterator, samplers_state_dict)
-            return batch
-
-        return apply_to_collection(loader_iters, Iterator, next_fn)
+        return apply_to_collection(loader_iters, Iterator, next)
 
     @staticmethod
     def create_loader_iters(
         loaders: Union[Any, Iterator, Sequence, Mapping]
     ) -> Union[Any, Iterator, Sequence, Mapping]:
-        """
-        Create and return a collection of iterators from loaders.
+        """Create and return a collection of iterators from loaders.
 
         Args:
             loaders: a collections of loaders
@@ -653,25 +583,3 @@ def _nested_calc_num_data(data: Union[Mapping, Sequence], compute_func: Callable
             new_data.append(x)
 
     return compute_func(new_data)
-
-
-def prefetch_iterator(iterable: Iterable) -> Generator[Tuple[Any, bool], None, None]:
-    """
-    Returns an iterator that pre-fetches and caches the next item.
-    The values are passed through from the given iterable with an added boolean indicating if this is the last item.
-    See `https://stackoverflow.com/a/1630350 <https://stackoverflow.com/a/1630350>`_
-    """
-    it = iter(iterable)
-
-    try:
-        # the iterator may be empty from the beginning
-        last = next(it)
-    except StopIteration:
-        return
-
-    for val in it:
-        # yield last and has next
-        yield last, False
-        last = val
-    # yield last, no longer has next
-    yield last, True
