@@ -14,12 +14,16 @@
 
 
 class MisconfigurationException(Exception):
-    """
-    Exception used to inform users of mis-use with PyTorch Lightning
-    """
+    """Exception used to inform users of mis-use with PyTorch Lightning."""
 
 
 class DeadlockDetectedException(Exception):
-    """
-    Exception used when a deadlock has been detected and processes are being killed
+    """Exception used when a deadlock has been detected and processes are being killed."""
+
+
+class ExitGracefullyException(Exception):
+    """Exception used when a ``signal.SIGUSR1`` is sent to the process.
+
+    This signals Lightning to try to create a fault-tolerance checkpoint once the current batch or epoch is reached
+    (assuming it can be done under 30 sec). After the checkpoint is saved, Lightning will exit.
     """
