@@ -11,7 +11,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-"""Helper functions to help with reproducibility of models. """
+"""Helper functions to help with reproducibility of models."""
 
 import logging
 import os
@@ -28,10 +28,8 @@ log = logging.getLogger(__name__)
 
 
 def seed_everything(seed: Optional[int] = None, workers: bool = False) -> int:
-    """
-    Function that sets seed for pseudo-random number generators in:
-    pytorch, numpy, python.random
-    In addition, sets the following environment variables:
+    """Function that sets seed for pseudo-random number generators in: pytorch, numpy, python.random In addition,
+    sets the following environment variables:
 
     - `PL_GLOBAL_SEED`: will be passed to spawned subprocesses (e.g. ddp_spawn backend).
     - `PL_SEED_WORKERS`: (optional) is set to 1 if ``workers=True``.
@@ -48,13 +46,19 @@ def seed_everything(seed: Optional[int] = None, workers: bool = False) -> int:
     max_seed_value = np.iinfo(np.uint32).max
     min_seed_value = np.iinfo(np.uint32).min
 
-    try:
-        if seed is None:
-            seed = os.environ.get("PL_GLOBAL_SEED")
+    if seed is None:
+        env_seed = os.environ.get("PL_GLOBAL_SEED")
+        if env_seed is None:
+            seed = _select_seed_randomly(min_seed_value, max_seed_value)
+            rank_zero_warn(f"No seed found, seed set to {seed}")
+        else:
+            try:
+                seed = int(env_seed)
+            except ValueError:
+                seed = _select_seed_randomly(min_seed_value, max_seed_value)
+                rank_zero_warn(f"Invalid seed found: {repr(env_seed)}, seed set to {seed}")
+    elif not isinstance(seed, int):
         seed = int(seed)
-    except (TypeError, ValueError):
-        seed = _select_seed_randomly(min_seed_value, max_seed_value)
-        rank_zero_warn(f"No correct seed found, seed set to {seed}")
 
     if not (min_seed_value <= seed <= max_seed_value):
         rank_zero_warn(f"{seed} is not in bounds, numpy accepts from {min_seed_value} to {max_seed_value}")
@@ -79,20 +83,20 @@ def _select_seed_randomly(min_seed_value: int = 0, max_seed_value: int = 255) ->
 
 
 def reset_seed() -> None:
-    """
-    Reset the seed to the value that :func:`pytorch_lightning.utilities.seed.seed_everything` previously set.
+    """Reset the seed to the value that :func:`pytorch_lightning.utilities.seed.seed_everything` previously set.
+
     If :func:`pytorch_lightning.utilities.seed.seed_everything` is unused, this function will do nothing.
     """
     seed = os.environ.get("PL_GLOBAL_SEED", None)
-    workers = os.environ.get("PL_SEED_WORKERS", False)
+    workers = os.environ.get("PL_SEED_WORKERS", "0")
     if seed is not None:
-        seed_everything(int(seed), workers=bool(workers))
+        seed_everything(int(seed), workers=bool(int(workers)))
 
 
-def pl_worker_init_function(worker_id: int, rank: Optional = None) -> None:  # pragma: no cover
-    """
-    The worker_init_fn that Lightning automatically adds to your dataloader if you previously set
-    set the seed with ``seed_everything(seed, workers=True)``.
+def pl_worker_init_function(worker_id: int, rank: Optional[int] = None) -> None:  # pragma: no cover
+    """The worker_init_fn that Lightning automatically adds to your dataloader if you previously set set the seed
+    with ``seed_everything(seed, workers=True)``.
+
     See also the PyTorch documentation on
     `randomness in DataLoaders <https://pytorch.org/docs/stable/notes/randomness.html#dataloader>`_.
     """
